# Generated by Django 5.2.9 on 2026-05-04 15:49
from email.policy import default

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('console_sessions', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='session',
            name='credential_id',
            field=models.TextField(null=True, blank=True, default=None),
        ),
        migrations.AddField(
            model_name='session',
            name='last_mfa',
            field=models.DateTimeField(null=True),
        ),
    ]
