CREDENTIAL_ID_SESSION_KEY = "_console_sessions_credential_id"
IP_SESSION_KEY = "_console_sessions_ip"
LAST_MFA_SESSION_KEY = "_console_sessions_last_mfa"
SESSION_LAST_MODIFIED_KEY = "_console_sessions_last_modified"
USER_AGENT_SESSION_KEY = "_console_sessions_user_agent"
